package X10;

use Data::Dumper;
use File::Basename;
use FileHandle;
use POSIX;

use strict;

use X10::DeviceList;
use X10::Log;
use X10::Macro;
use X10::SchedEvent;
use X10::Scheduler;
use X10::Server;

use vars qw($VERSION);

$VERSION = 0.05;

my $object;

sub init
{
   my $type = shift;

   my $self = bless { @_ }, $type;

   unshift @INC, $self->{base_dir} if exists $self->{base_dir};

   unless ($self->{controller_type})
   {
      warn "Interface type must be specified\n";
      return undef;
   }

   x10_log('notice', "%s service starting", $self->{controller_type});

   my $controller = $self->{controller_type};

   if (eval "require $controller")
   {
      $controller->import;	# just in case
   }
   else
   {
      die "Can't load module for $controller: ", $@;
   }

   $self->{controller} = $controller->new(
	port => $self->{controller_device},
	debug => $self->{debug},
	);

   $self->{controller}->register_listener(
	sub { x10_log('info', "Event: %s", $_[0]->as_string) }
	);

   if (exists $self->{deviceconfig})
   {
      # load device config
      $self->{devices} = new X10::DeviceList(
	base_dir => $self->{base_dir},
	configfile => $self->{deviceconfig},
	controller => $self->{controller},
	debug => $self->{debug},
	);
   }

   if (exists $self->{schedulerconfig})
   {
      $self->{scheduler} = new X10::Scheduler(
	base_dir => $self->{base_dir},
	configfile => $self->{schedulerconfig},
	controller => $self->{controller},
	debug => $self->{debug},
	latitude => 38.74274,
	longitude => -90.560143,
	);

      unless ($self->{scheduler})
      {
         warn "Problem creating scheduler";
         return undef;
      }

   }

   if (exists $self->{server_port})
   {
      $self->{server} = new X10::Server(
	controller => $self->{controller},
	debug => $self->{debug},
	server_port => $self->{server_port},
	);

      unless ($self->{server})
      {
         warn "Problem creating network server";
         return undef;
      }
   }

   $object = $self;
}


sub run
{
   my $self = $object;

   # this method plans to never return...

   $self->{running} = 1;

   x10_log('notice', "Running...");

   # $SIG{'INT'} = sub { $self->{running} = 0; };
   # $SIG{'TERM'} = sub { $self->{running} = 0; };
   $SIG{'INT'} = sub { x10_log('notice', 'Shutting down on INT'); exit; };
   $SIG{'TERM'} = sub { x10_log('notice', 'Shutting down on TERM'); exit; };

   my $next_wakeup = 0;

   X10RUNMAINLOOP:
   while ($self->{running})
   {
      x10_log('debug', "Entering mainloop");

      my %fdindex;
      foreach my $module (
		grep { exists $self->{$_} } qw(controller server devices)
		)
      {
         foreach my $fd ($self->{$module}->select_fds)
         {
            $fdindex{$fd} = $self->{$module};
         }
      }

      my $rfd = '';
      foreach (keys %fdindex) { vec($rfd, $_, 1) = 1; }

      # done setting up FD array

      # figure out if we have to wake up at a certain time:

      my $timeout = undef;

      if ($self->{scheduler})
      {
         my $next_event = $self->{scheduler}->next_event;

         if ($next_event)
         {
            $timeout = $next_event->next_time - int(time);
            $timeout = 0 if ($timeout < 0);
         }

         if ( (defined $timeout) && $next_event && $next_event->next_time != $next_wakeup)
         {
            $next_wakeup = $next_event->next_time;
            x10_log('info', "Next Scheduled Event: %s at %s (%s seconds away)",
		$self->{scheduler}->next_event->description,
		strftime("%a %b %e %H:%M %Y", localtime($next_event->next_time)),
		$timeout,
		);
         }

      }

      # done calculating wakeup time

      x10_log('debug', "selecting:  readers=(%s), timeout=%s", join(', ', keys %fdindex), $timeout || 0);

      my $readers;
      my $fdcount = select($readers=$rfd, undef, undef, $timeout);

      if ($fdcount > 0)
      {
         x10_log('debug', "Got %s FDs to handle", $fdcount);

         foreach my $fd (keys %fdindex)
         {
            if (vec($readers, $fd, 1))
            {
               x10_log('debug', "Processing input on FD %s (%s)", $fd, ref $fdindex{$fd});
               $fdindex{$fd}->handle_input($fd);
            }
         }

      }
      elsif ($fdcount < 0 && $! != 4)	# ignore Interrupted System Call
      {
         x10_log('error', "Error %d in select: %s", $!, $!);
      }

      if ($self->{scheduler})
      {
         $self->{scheduler}->run;
      }

   }

   x10_log('notice', "%s service shutting down", $self->{controller_type});

}

1;

