package X10::ClientConnection;

use IO::Select;
use IO::Socket;

use strict;

use X10::Log;
use X10::Event;
use X10::EventList;

sub new
{
   my $package = shift;
   my $socket = shift;
   my $controller = shift;

   return undef unless $controller;

   my $self = bless {
	socket => $socket,
	controller => $controller,
	buffer => '',
	state => 'new',
	}, $package;

   $self->{clienthost} = gethostbyaddr($self->{socket}->peeraddr, AF_INET)
	|| $self->{socket}->peerhost;

   x10_log('info', "New socket from %s", $self->{clienthost});

   $self->{controller}->register_listener( sub { $self->handle_event(shift) } );

   return $self;
}


sub handle_event
{
   my $self = shift;
   my $event = shift;

   $self->send_event($event);
}

sub handle_input
{
   my $self = shift;

   while ($self->{socket} && IO::Select->new($self->{socket})->can_read(0))
   {
      my $len = $self->{socket}->sysread($self->{buffer}, 100, length($self->{buffer}));

      if ($len == 0)
      {
         # x10_log('notice', "EOF on socket from %s", $self->{clienthost});
         $self->close;
      }

      if (length($self->{buffer}) >= 1000)
      {
         $self->close;
      }

      $self->run_buffer;

   }

}

sub run_buffer
{
   my $self = shift;

   return unless $self->{buffer};

   $self->{buffer} =~ s/^[\s\r\n]+//;		# strip off leading newlines & whitespace
   $self->{buffer} = uc($self->{buffer});

   while (my ($cmd, $rest) = $self->{buffer} =~ m/^(.*?)\s*[\r\n]+(.*)$/)
   {
      $self->{buffer} = $rest;
      $self->do_command($cmd);
   }
}

sub is_open
{
   my $self = shift;
   return $self->{socket} ? 1 : 0;
}

sub close
{
   my $self = shift;
   x10_log('info', "Closing socket from %s", $self->{clienthost});
   # $self->{socket}->close;
   delete $self->{socket};

   $self->run_buffer;
}

sub do_command
{
   my $self = shift;
   my $cmd = shift;

   x10_log('debug', "Got line from client: %s", $cmd);

   my @words = split(/\s+/, $cmd);

   if ($words[0] eq 'QUIT')
   {
      $self->close;
   }
   elsif ($words[0] eq 'SEND')
   {
      shift @words;

      my $eventlist = new X10::EventList(@words);

      if ($eventlist)
      {
         x10_log('debug', "Sending command from client: %s", $eventlist->short_string);
         $self->{controller}->send($eventlist);
      }
      else
      {
         my $message = "Not Understood\r\n";
         $self->{socket}->syswrite($message, length($message));
      }

   }
   else
   {
      x10_log('error', "Unrecognized Command: %s", $cmd);
   }
}

sub send_event
{
   my $self = shift;
   my $event = shift;

   return unless $self->{socket};

   my $line = sprintf("GOT %s\r\n", $event->short_string);

   $self->{socket}->syswrite($line, length($line));
}


1;

