package X10::DeviceList;

# this package implements a macro processor using an event callback mechanism

use Data::Dumper;

use strict;

use X10::Event;
use X10::Log;

sub new
{
   my $type = shift;

   my $self = bless { @_ }, $type;

   return undef unless $self->{controller};

   x10_log('debug', "Constructing Device List");

   $self->{controller}->register_listener($self->event_callback);
   $self->{devices} = {};

   $self->read_config;

   return $self;
}

###

sub read_config
{
   my $self = shift;

   return unless $self->{configfile};

   $self->{config_lastmod} = 0 unless exists $self->{config_lastmod};

   x10_log('debug', "Checking %s config file...", __PACKAGE__);

   if ($self->{config_lastmod} == 0 || (stat($INC{$self->{configfile}}))[9] > $self->{config_lastmod})
   {
      my $saved_config = $self->{devices};
      $self->{devices} = {};
      $self->{config_lastmod} = time;

      x10_log('notice', "Reading device config file...");

      delete $INC{$self->{configfile}} if (exists $INC{$self->{configfile}});

      my $config = eval { require $self->{configfile} };

      if ($@)
      {
         x10_log('err', "Device Config Read Failed: $@");
         $self->{devices} = $saved_config;
         return;
      }
      elsif ('HASH' ne ref $config)
      {
         x10_log('err', "Device Config Read Failed: Not a HASHREF");
         $self->{devices} = $saved_config;
         return;
      }

      foreach (keys %$config)
      {
         $self->add( $_ => $config->{$_} );
      }

      x10_log('notice', "Got %s devices", scalar keys %$config);

      # print STDERR Dumper($self->{devices});
   }
}

sub add
{
   my $self = shift;
   my ($key, $device) = @_;

   # my $nkey = X10::Event->new($key)->as_string;

   my ($hc, $ad) = $key =~ /^([a-p])(\d{1,2})$/i;
   $key = sprintf("%1s%02s", uc $hc, 1 * $ad);

   x10_log('info', "Replacing old device at %s", $key)
	if exists $self->{devices}->{$key};

   # set here so we don't have to set twice in config file
   $device->controller($self->{controller});
   $device->address($key);

   $self->{devices}->{$key} = $device;
   $self->{devices}->{$key}->controller($self->{controller});
}

sub setup
{
   my $self = shift;

   if (@_)
   {
      while (my $key = shift)
      {
         my $nkey = X10::Event->new($key)->as_string;

         if ($nkey)
         {
            $self->{devices}->{$nkey} = shift;
            $self->{devices}->{$nkey}->controller($self->{controller});
         }
         else
         {
            x10_log('info', "Throwing away device for ", $key);
            shift;
         }
      }

   }

   return $self->{devices};
}

sub event_callback
{
   my $self = shift;
   return sub { $self->handle_event(shift) };
}

sub handle_event
{
   my $self = shift;
   my $event = shift;

   $self->read_config;	# check for updated config file...

   if ($event->unit_code eq 'ALL')
   {
      my @affected =
	sort { $a->unit_code <=> $b->unit_code }
	grep { $_->house_code eq $event->house_code }
	values %{$self->{devices}};

      foreach (@affected)
      {
         $_->got_event($event->func);
      }
   }
   elsif ($event->unit_code eq 'LIGHTS')
   {
      my @affected =
	sort { $a->unit_code <=> $b->unit_code }
	grep { $_->is_light }
	grep { $_->house_code eq $event->house_code }
	values %{$self->{devices}};

      foreach (@affected)
      {
         $_->got_event($event->func);
      }
   }
   else
   {
      if (exists $self->{devices}->{$event->address})
      {
         $self->{devices}->{$event->address}->got_event($event->func);
      }
      else
      {
         x10_log('notice', "Event: %s", $event->as_string);
      }
   }

}

sub select_fds
{
   return ();
}


1;

