package X10::Network;
use vars qw(@ISA);
@ISA = qw(X10::Controller);

# this is an X10::Controller-type object, used to control X10 devices
# via a process running an X10::Server object somewhere else

use IO::Select;
use IO::Socket;

use strict;

use X10::Controller;
use X10::Event;
use X10::EventList;
use X10::Log;

### constructors

sub new
{
   my $type = shift;

   my $self = new X10::Controller( @_ );
   bless $self, $type;

   $self->{socket} = new IO::Socket(
        Domain => &AF_INET,
        Proto => 'tcp',
        PeerAddr => $self->{server} || 'x10',
        PeerPort => $self->{server_port} || 2020,
        );

   $self->{buffer} = '';

   unless ($self->{socket})
   {
      warn "Problem connecting socket: ", $!;
      return undef;
   }

   return $self;
}


### public methods (most overriding parent class)

sub select_fds
{
   my $self = shift;
   return ($self->{socket}->fileno);
}

sub send
{
   my $self = shift;
   foreach (@_)
   {
      $self->send_one($_);
   }
}

sub send_one
{
   my $self = shift;
   my $event = shift;

   my $packet = sprintf("SEND %s\r\n", $event->short_string);

   x10_log('debug', "Sending packet: %s", $packet);

   $self->{socket}->syswrite($packet, length($packet));

   # my $line = $self->{socket}->getline;
}

sub handle_input
{
   my $self = shift;

   while ($self->{socket} && IO::Select->new($self->{socket})->can_read(0))
   {
      $self->{socket}->sysread($self->{buffer}, 100, length($self->{buffer}));

      $self->run_buffer;
   }
}

sub run_buffer
{
   my $self = shift;

   return unless $self->{buffer};

   $self->{buffer} = uc($self->{buffer});

   while (my ($cmd, $rest) = $self->{buffer} =~ m/^\s*(.*?)\s*[\r\n]+(.*)$/)
   {
      $self->{buffer} = $rest;

      $self->do_command($cmd);
   }
}

sub do_command
{
   my $self = shift;
   my $cmd = shift;

   x10_log('debug', "Got: %s", $cmd);

   my @words = split(/\s+/, $cmd);

   if ($words[0] eq 'GOT')
   {
      shift @words;
   
      my $eventlist = new X10::EventList(@words);

      if ($eventlist)
      {
         x10_log('debug', "Got %s", $eventlist->short_string);
         $self->got_event($eventlist);
      }
      else
      {
         my $message = "Not Understood\r\n";
         $self->{socket}->syswrite($message, length($message));
      }
   
   }
   else
   {
      x10_log('error', "Unrecognized Command: %s", $cmd);
   }
}

### mostly-private methods...

sub DESTROY
{
   my $self = shift;

   my $command = 'QUIT\r\n';

   $self->{socket}->syswrite($command, length($command));

   $self->{socket}->close;
}

### utility functions -- not called as methods



1;

