package X10::SchedEvent;

use Astro::Sunrise;
use POSIX;
use Time::ParseDate;
use Time::Zone;

use strict;

use X10::Log;

sub new
{
   my $type = shift;

   my $self = bless { @_ }, $type;

   unless ($self->{macro} && $self->{macro}->isa('X10::Macro'))
   {
      warn "No macro sent to create SchedEvent";
      return undef;
   }

   $self->{next_time} = int(time);
   $self->reschedule;
   $self->{last_time} = 0;

   return $self;
}



sub run
{
   my $self = shift;

   x10_log('info', "Scheduled Event: %s",
	$self->{description} || 'unnamed event',
	);

   $self->{macro}->run;
}

sub reschedule
{
   my $self = shift;

   $self->{last_time} = $self->{next_time} || 0;

   my $current = $self->{last_time} || time;
   my $next;

   if (!exists $self->{repeat_type} || $self->{repeat_type} eq 'none')
   {
      return 0;
   }
   elsif ($self->{repeat_type} eq 'day')
   {
      my $new = parsedate(sprintf("today %s", $self->time('today')));

      if ($new <= time || $new <= $current)
      {
         $new = parsedate(sprintf("tomorrow %s", $self->time('tomorrow')));
      }

      # check DOW loop here...
      if (exists $self->{dowmask} && $self->{dowmask} > 0)
      {
         my @newarray = localtime($new);

         while ( ! ( (1 << $newarray[6]) & $self->{dowmask} ) )
         {
            x10_log('debug', "Skipping %s...", strftime("%a %b %e %Y", @newarray));

            $newarray[3] += 1;		# add one day
            @newarray = localtime(mktime(@newarray));	# normalize

            # find new time on that day...

            my $datestr = strftime("%a %b %e %Y", @newarray);

            my $time = $self->time($datestr);

            $new = parsedate(sprintf("%s %s", $time, $datestr), WHOLE => 1);
            @newarray = localtime($new);

         }

      }

      $next = $new;
   }
   else
   {
      x10_log('warning', "Unsupported repeat type: %s", $self->{repeat_type});
   }


   $self->{next_time} = $next;

   return 1;
}

sub next_time
{
   my $self = shift;

   return $self->{next_time};
}

sub controller
{
   my $self = shift;

   if (@_)
   {
      $self->{macro}->controller(shift);
   }

   return $self->{macro}->controller;
}

sub description
{
   my $self = shift;

   if (@_)
   {
      $self->{description} = shift;
   }

   return $self->{description};
}

sub time
{
   my $self = shift;
   my $date = shift;

   # offsets assumed not to force time across day boundaries...

   my $time;
   my $sign;
   my $offset;

   if ($self->{time} =~ /^\d?\d:\d\d$/)
   {
      $time = $self->{time};
      $offset = 0;
   }

   # h = -6 degrees: Civil twilight (one can no longer read outside without artificial illumination)
   # h = -12 degrees: Nautical twilight (navigation using a sea horizon no longer possible)
   # h = -15 degrees: Amateur astronomical twilight (the sky is dark enough for most astronomical observations)
   # h = -18 degrees: Astronomical twilight (the sky is completely dark)

   elsif ($self->{time} =~ /^(\w+)(\s*([+-])\s*(\d+))?$/)
   {
      my $type = $1;
      $sign = ($3 eq '-') ? -1 : 1;
      $offset = $4 || 0;

      if ($type eq 'sunrise')
      {
         $time = $self->sunrise_time( date => $date );
      }
      elsif ($type eq 'sunset')
      {
         $time = $self->sunset_time( date => $date );
      }
      elsif ($type eq 'civiltwilight_set')
      {
         $time = $self->sunset_time( date => $date, h => -6 );
      }
      elsif ($type eq 'nauticaltwilight_set')
      {
         $time = $self->sunset_time( date => $date, h => -12 );
      }
      elsif ($type eq 'astrotwilight_set')
      {
         $time = $self->sunset_time( date => $date, h => -18 );
      }
      else
      {
         x10_log('warning', "Unknown time string: %s", $self->{time});
         return undef;
      }

   }
   else
   {
      x10_log('warning', "Unknown time string: %s", $self->{time});
      return undef;
   }

   return $time unless $offset;

   my ($hour, $minute) = $time =~ /^(\d?\d):(\d\d)$/;

   $minute += $sign * $offset;

   while ($minute >= 60)
   {
      $minute -= 60;
      $hour++;
   }

   while ($minute < 0)
   {
      $minute += 60;
      $hour--;
   }

   return sprintf("%s:%02s", $hour, $minute);
}


################

## INTERNAL USE ONLY

sub times
{
   my $self = shift;

   my %params = @_;

   my $type = $params{type};
   my $date = $params{date};

   # my @elements = localtime(parsedate($date, NO_RELATIVE => 1));
   my @elements = localtime(parsedate($date));

   my ($yyyy, $mm, $dd, $dst) = @elements[5,4,3,8];
   $yyyy += 1900;
   $mm += 1;

   my $offset = tz_offset('CST') / 3600;
   my $lat = 38.715153;
   my $lon = -90.651056;

   my $h = $params{h} || -0.866;

   my @times = sunrise(
	$yyyy, $mm, $dd,
	$lon, $lat,
	$offset, $dst,
	$h,
	);

   return \@times;
}

sub sunrise_time
{
   my $self = shift;
   return $self->times(@_)->[0];
}

sub sunset_time
{
   my $self = shift;
   return $self->times(@_)->[1];
}



1;

